SystemOrganization addCategory: #Brltty!
SystemOrganization addCategory: #'Brltty-Tests'!

TestCase subclass: #BrlttyTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Brltty-Tests'!

!BrlttyTests methodsFor: 'tests' stamp: 'ThierryGoubier 4/5/2012 21:49'!
testBrlttyAuth
	"Test reading the version and the auth of brltty once connected."

	| s d version auth k r t |
	s := BrlttyConnection current.
	[ s connect ]
		on: ConnectionTimedOut
		do: [ :ex | 
			Transcript
				show: 'Timed out trying to connect to brltty.';
				cr.
			s disconnect.
			^ self ].
	version := s readVersion.
	Transcript
		show: 'Brltty connect replies: (version = ' , version printString , ')';
		cr.
	s sendVersion.
	auth := s readAuth asCharacter.
	Transcript
		show: 'Brltty connect replies: (authorisation = ' , auth printString , ')';
		cr.
	k := (FileStream readOnlyFileNamed: '/etc/brlapi.key') contentsOfEntireFile.
	s sendAuthKey: k.
	r := s nextUint32.
	t := s nextUint32.
	Transcript show: 'Brltty auth replies: (length = ' , r printString , ', type = ', t printString, ')';
		cr.
	s disconnect! !

!BrlttyTests methodsFor: 'tests' stamp: 'ThierryGoubier 4/5/2012 21:50'!
testConnectToBrltty
	"Test the connection to brltty."

	| s |
	s := BrlttyConnection current.
	[ s connect ]
		on: ConnectionTimedOut
		do: [ :ex | 
			Transcript
				show: 'Timed out trying to connect to brltty.';
				cr.
			s disconnect.
			^ self ].
	s disconnect! !

!BrlttyTests methodsFor: 'tests' stamp: 'ThierryGoubier 4/5/2012 21:50'!
testReadBrlttyVersion
	"Test reading the version of brltty once connected."

	| s d sz type version |
	s := BrlttyConnection current.
	[ s connect ]
		on: ConnectionTimedOut
		do: [ :ex | 
			Transcript
				show: 'Timed out trying to connect to brltty.';
				cr.
			s disconnect.
			^ self ].
	version := s readVersion.
	Transcript
		show: 'Brltty connect replies: (version = ' , version printString , ')';
		cr.
	s disconnect! !

!BrlttyTests methodsFor: 'tests' stamp: 'ThierryGoubier 4/5/2012 21:51'!
testReadBrlttyVersionAndAuth
	"Test reading the version and the auth of brltty once connected."

	| s d version auth |
	s := BrlttyConnection current.
	[ s connect ]
		on: ConnectionTimedOut
		do: [ :ex | 
			Transcript
				show: 'Timed out trying to connect to brltty.';
				cr.
			s disconnect.
			^ self ].
	version := s readVersion.
	Transcript
		show: 'Brltty connect replies: (version = ' , version printString , ')';
		cr.
	s sendVersion.
	auth := s readAuth asCharacter.
	Transcript
		show: 'Brltty connect replies: (authorisation = ' , auth printString , ')';
		cr.
	s disconnect! !

Object subclass: #BrlttyConnection
	instanceVariableNames: 'socket'
	classVariableNames: 'current'
	poolDictionaries: ''
	category: 'Brltty'!

!BrlttyConnection commentStamp: 'ThierryGoubier 4/6/2012 13:45' prior: 0!
BrlttyConnection holds a connection to the brltty server port. It holds a SocketStream instance.

The BrlttyConnection object should be designed so that :
- The object is in charge of the connect and authentification phase, as well as disconnects and reconnects (? Quitting, restarting the image). The connect also does the switch to tty mode (and disconnect from it when losing the focus).
- Once connected, the morph which has the focus sends it's name to the Brltty connection object.
	- First attempt : the connection reduces the string to the keyboard size
	- Second attempt : the connection handles a tty-like approach to allow the keyboard to see the whole of the string (then, track the keyboard keys.)
	- Text areas have a different behavior and follow their cursors to send the string part around the cursor, in agreement with the braille keyboard size (retrieved through the connection)
- It's necessary to put the BrlttyConnection object in it's own process.
- Behavior on error or connection loss has to be defined.

IHM behavior (Ph DORE)
- Display the window title first (and its type: dialog / window)
- Display the control type and name next
- Display the control contents (around the cursor or full if appropriate)
- Add a sense of the control position.
- Keyboard keys allows for
	- Moving along the line if the line is longer than the braille display
	- Move up and down and navigate.
	- Move the mouse pointer (used to have this on keyboards ?)
	- In some keyboards, braille key entry (replace a normal keyboard)
	- Jump to specific parts (window title)

Instance Variables:
	socket	<SocketStream>!

!BrlttyConnection class methodsFor: 'accessing' stamp: 'ThierryGoubier 4/3/2012 17:06'!
current
	current isNil
		ifTrue: [ current := self new ].
	^ current! !

!BrlttyConnection class methodsFor: 'accessing' stamp: 'ThierryGoubier 4/3/2012 11:03'!
current: anObject
	current := anObject! !

!BrlttyConnection class methodsFor: 'comments' stamp: 'ThierryGoubier 4/4/2012 15:19'!
elBrltty
	";;; brltty.el --- Interface to BrlTTY

;; Copyright (C) 2004, 2005 Brailcom, o.p.s.

;; Author: Milan Zamazal <pdm@brailcom.org>

;; COPYRIGHT NOTICE
;;
;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2 of the License, or
;; (at your option) any later version.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program; if not, write to the Free Software
;; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

;;; Commentary:

;;; Code:


(require 'cl)

(require 'speechd-common)


;;; User configuration and commands


(defgroup brltty ()
  ""BrlTTY interface.""
  :group 'speechd-el)
  
(defcustom brltty-default-host ""localhost""
  ""Default BrlTTY host to connect to.""
  :type 'string
  :group 'brltty)

(defcustom brltty-default-port 35751
  ""Default BrlTTY port to connect to.""
  :type 'integer
  :group 'brltty)

(defcustom brltty-authentication-file ""/etc/brlapi.key""
  ""File containing the BrlAPI authentication key.""
  :type '(file :must-match t)
  :group 'brltty)

(defcustom brltty-coding 'iso-8859-1
  ""Coding in which texts should be sent to BrlTTY.""
  :type 'coding-system
  :group 'brltty)

(defcustom brltty-tty (car (read-from-string
                            (or (let ((value (ignore-errors
                                               (shell-command-to-string ""xprop -root XFree86_VT""))))
                                  (and value
                                       (string-match ""= *\\([0-9]+\\)"" value)
                                       (match-string 1 value)))
                                (getenv ""CONTROLVT"")
                                ""0"")))
  ""Number of the Linux console on which brltty.el runs.
The default value is taken from the XFree86_VT root window property or, if not
available, from the  environment variable CONTROLVT.""
  :type 'integer
  :group 'brltty)

(defcustom brltty-timeout 3
  ""Maximum number of seconds to wait for a BrlTTY answer.""
  :type 'integer
  :group 'brltty)


;;; Internal functions and data


(defconst brltty--emacs-accept-ok
  (condition-case _ (progn (accept-process-output nil 0 0 1) t) (error)))

(defconst brltty--emacs-process-ok (fboundp 'process-put))

(defconst brltty--protocol-version 7)

(defconst brltty--packet-types
  '(;; commands
    (authkey . ?K)
    (gettty . ?t)
    (leavetty . ?L)
    (getdisplaysize . ?s)
    (write . ?w)
    ;; answers
    (ack . ?A)
    (err . ?e)
    (key . ?k)))

(defun brltty--add-answer (connection answer)
  (setf (brltty--connection-answers connection)
        (append (brltty--connection-answers connection) (list answer)))
  answer)

(defun brltty--next-answer (connection)
  (pop (brltty--connection-answers connection)))

(defstruct brltty--connection
  process
  (display-width nil)
  (output "")
  (key-handler nil)
  (answers '()))

(unless brltty--emacs-process-ok
  (defvar brltty--process-connections '()))

(defun brltty--open-connection (host port key-handler)
  (let ((process (open-network-stream ""brltty"" nil
                                      (or host brltty-default-host)
                                      (or port brltty-default-port))))
    (when process
      (set-process-coding-system process 'raw-text-unix 'raw-text-unix)
      (if (fboundp 'set-process-query-on-exit-flag)
          (set-process-query-on-exit-flag process nil)
        (process-kill-without-query process))
      (let ((connection (make-brltty--connection :process process
                                                 :key-handler key-handler)))
        (if brltty--emacs-process-ok
            (process-put process 'brltty-connection connection)
          (push (cons process connection) brltty--process-connections))
        (set-process-filter process #'brltty--process-filter)
        connection))))

(defun brltty--process-connection (process)
  (if brltty--emacs-process-ok
      (process-get process 'brltty-connection)
    (cdr (assq process brltty--process-connections))))

(defun brltty--disable-connection (connection error)
  (let ((process (brltty--connection-process connection)))
    (when process
      (delete-process process)
      (setf (brltty--connection-process connection) nil)))
  (error ""Error in communication with BrlTTY: %s"" error))

(defun brltty--process-filter* (process output)
  (let ((connection (brltty--process-connection process)))
    (setf (brltty--connection-output connection)
          (concat (brltty--connection-output connection) output))))

(defun brltty--process-filter (process output)
  (set-process-filter process #'brltty--process-filter*)
  (unwind-protect
      (let ((connection (brltty--process-connection process)))
        (brltty--process-filter* process output)
        (while (not (string= (brltty--connection-output connection) ""))
          (brltty--read-input connection)))
    (set-process-filter process #'brltty--process-filter)))

(defun brltty--accept-process-output (process)
  (if brltty--emacs-accept-ok
      (accept-process-output process brltty-timeout nil 1)
    (accept-process-output process brltty-timeout)))

(defun brltty--read-integer (string)
  (+ (* 256 256 256 (aref string 0)) (* 256 256 (aref string 1))
     (* 256 (aref string 2)) (aref string 3)))

(defun brltty--read-packet (connection)
  (condition-case err
      (let ((process (brltty--connection-process connection)))
        (flet ((read-enough-output (size)
                 (while (< (length (brltty--connection-output connection)) size)
                   (brltty--accept-process-output process)))
               (read-integer ()
                 (let ((output (brltty--connection-output connection)))
                   (prog1 (brltty--read-integer output)
                     (setf (brltty--connection-output connection)
                           (substring output 4))))))
          (read-enough-output 8)
          (let* ((size (read-integer))
                 (type (car (rassoc (read-integer) brltty--packet-types)))
                 (data (progn
                         (read-enough-output size)
                         (let ((output (brltty--connection-output connection)))
                           (setf (brltty--connection-output connection)
                                 (substring output size))
                           (substring output 0 size)))))
            (cons type data))))
    (error
     (brltty--disable-connection connection err))))

(defun brltty--read-input (connection)
  (destructuring-bind (type . data) (brltty--read-packet connection)
    (case type
     (err
      (error (format ""BrlTTY error %d: %s"" (brltty--read-integer data) data)))
     (key
      (let ((handler (brltty--connection-key-handler connection)))
        (when handler
          (funcall handler (brltty--read-integer data)))))
     (nil
      ;; unknown packet type -- ignore
      )
     (getdisplaysize
      (brltty--add-answer connection
                          (list type
                                (brltty--read-integer (substring data 0 4))
                                (brltty--read-integer (substring data 4 8)))))
     (t
      (brltty--add-answer connection (list type data))))))

(defun brltty--read-answer (connection packet-id)
  (let ((process (brltty--connection-process connection))
        (answer '(nothing-yet)))
    (while (and answer (not (eq (car answer) packet-id)))
      ;; The answer may be already present in answers
      (setq answer (brltty--next-answer connection))
      (unless answer
        (brltty--accept-process-output process)
        (setq answer (brltty--next-answer connection))))
    (unless answer
      (error ""BrlTTY answer not received""))
    (cdr answer)))

(defvar foo nil)
(defun brltty--send-packet (connection answer packet-id &rest data-list)
  (let ((length (reduce #'+ data-list :initial-value 0
                        :key #'(lambda (data)
                                 (cond 
				  ((integerp data) 4)
				  (t (length data))))))
        (process (brltty--connection-process connection)))
    (when process
      (with-speechd-coding-protection
        (condition-case err
            (flet ((send-integer (n)
		     (push (apply #'format ""%c%c%c%c""
                             (funcall #'reverse
                                      (loop for i from 1 to 4
                                            for x = n then (/ x 256)
                                            for rem = (% x 256)
                                            collect rem))) foo)
                     (process-send-string
                      process
                      (apply #'format ""%c%c%c%c""
                             (funcall #'reverse
                                      (loop for i from 1 to 4
                                            for x = n then (/ x 256)
                                            for rem = (% x 256)
                                            collect rem))))))
              (send-integer length)
              (send-integer (cdr (assoc packet-id brltty--packet-types)))
              (dolist (data data-list)
                (cond
                 ((integerp data)
                  (send-integer data))
                 ((vectorp data)
                  (dotimes (i (length data))
		    (push (format ""%c"" (aref data i)) foo)
                    (process-send-string process (format ""%c"" (aref data i)))))
                 (t
		  (push data foo)
                  (process-send-string process data))))
              (when answer
                (brltty--read-answer connection answer)))
          (error
           (brltty--disable-connection connection err)))))))

(defun brltty--authentication-key ()
  (with-temp-buffer
    (insert-file-contents brltty-authentication-file)
    (buffer-substring (point-min) (point-max))))


;;; Public functions and data


(defun brltty-open (&optional host port key-handler)
  ""Open and return connection to a BrlTTY server running on HOST and PORT.
If HOST or PORT is nil, `brltty-default-host' or `brltty-default-port' is used
respectively.""
  (condition-case err
      (let ((connection (brltty--open-connection host port key-handler)))
        (brltty--send-packet connection 'ack 'authkey
                             brltty--protocol-version
                             (brltty--authentication-key))
        (brltty--send-packet connection 'ack 'gettty 1 brltty-tty [0])
        connection)
    (error
     (message ""Error on opening BrlTTY connection: %s"" err)
     nil)))

(defun brltty-close (connection)
  ""Close BrlTTY CONNECTION.""
  (when connection
    (brltty--send-packet connection 'ack 'leavetty)
    (let ((process (brltty--connection-process connection)))
      (unless brltty--emacs-process-ok
        (setq brltty--process-connections
              (remove (assq process brltty--process-connections)
                      brltty--process-connections)))
      (when process
        (delete-process process)))))

(defun brltty-display-size (connection)
  ""Return the size of the display as the list (WIDTH HEIGHT)""
  (when connection
    (or (brltty--connection-display-width connection)
        (setf (brltty--connection-display-width connection)
              (brltty--send-packet connection 'getdisplaysize
                                   'getdisplaysize)))))

(defun brltty-write (connection text &optional cursor)
  ""Display TEXT in BrlTTY accessed through CONNECTION.
TEXT is encoded in the coding given by `brltty-coding' before it is sent.
CURSOR, if non-nil, is a position of the cursor on the display, starting
from 0.""
  (when connection
    (let ((display-width (car (brltty-display-size connection))))
      (when display-width
        (let* ((text* (if (> (length text) display-width)
                          (substring text 0 display-width)
                        ;; We must be careful with FORMAT because of formatting
                        ;; of TAB characters
                        (concat text
                                (format
                                 (format ""%%-%ds"" (- display-width
                                                     (length text)))
                                 ""))))
               (encoded-text (with-speechd-coding-protection
                              (encode-coding-string text* brltty-coding))))
          (brltty--send-packet connection nil 'write
                               38
                               1 display-width
                               (length encoded-text) encoded-text
                               ;; Cursor position may not be too high,
                               ;; otherwise BrlTTY breaks the connection
                               (if cursor
                                   (1+ (min cursor display-width))
                                 0)))))))


;;; Announce

(provide 'brltty)


;;; brltty.el ends here
"! !

!BrlttyConnection class methodsFor: 'comments' stamp: 'ThierryGoubier 4/5/2012 16:59'!
packetFormat
	"Here is the format for handling communication to the brltty server.

a packet format is :
uint32 size;
uint32 type;
char[size] data;

but data in itself can have other contents:
for example, a auth reply packet from the client is :
uint32 auth_type;
unsigned char[size -4] key;

#define BRLAPI_MAXPACKETSIZE 512

#define BRLAPI_PACKET_VERSION         'v'   /**< Version                     */
#define BRLAPI_PACKET_AUTH            'a'   /**< Authorization               */
#define BRLAPI_PACKET_GETDRIVERNAME   'n'   /**< Ask which driver is used    */
#define BRLAPI_PACKET_GETDISPLAYSIZE  's'   /**< Dimensions of brl display   */
#define BRLAPI_PACKET_ENTERTTYMODE    't'   /**< Asks for a specified tty    */
#define BRLAPI_PACKET_SETFOCUS        'F'   /**< Set current tty focus       */
#define BRLAPI_PACKET_LEAVETTYMODE    'L'   /**< Release the tty             */
#define BRLAPI_PACKET_KEY             'k'   /**< Braille key                 */
#define BRLAPI_PACKET_IGNOREKEYRANGES 'm'   /**< Mask key ranges             */
#define BRLAPI_PACKET_ACCEPTKEYRANGES 'u'   /**< Unmask key ranges           */
#define BRLAPI_PACKET_WRITE           'w'   /**< Write                       */
#define BRLAPI_PACKET_ENTERRAWMODE    '*'   /**< Enter in raw mode           */
#define BRLAPI_PACKET_LEAVERAWMODE    '#'   /**< Leave raw mode              */
#define BRLAPI_PACKET_PACKET          'p'   /**< Raw packets                 */
#define BRLAPI_PACKET_ACK             'A'   /**< Acknowledgement             */
#define BRLAPI_PACKET_ERROR           'e'   /**< non-fatal error             */
#define BRLAPI_PACKET_EXCEPTION       'E'   /**< Exception                   */
#define BRLAPI_PACKET_SUSPENDDRIVER   'S'   /**< Suspend driver              */
#define BRLAPI_PACKET_RESUMEDRIVER    'R'   /**< Resume driver               */
"! !

!BrlttyConnection methodsFor: 'brltty defines' stamp: 'ThierryGoubier 4/3/2012 21:58'!
brlapiAuthCredential
	^ $C asciiValue! !

!BrlttyConnection methodsFor: 'brltty defines' stamp: 'ThierryGoubier 4/3/2012 21:58'!
brlapiAuthKey
	^ $K asciiValue! !

!BrlttyConnection methodsFor: 'brltty defines' stamp: 'ThierryGoubier 4/3/2012 21:58'!
brlapiAuthNone
	^ $N asciiValue! !

!BrlttyConnection methodsFor: 'network' stamp: 'ThierryGoubier 4/3/2012 21:11'!
connect
	"Connect the instance to the right brltty port"
	
	socket notNil ifTrue: [^self].
	socket := SocketStream openConnectionToHost: NetNameResolver localHostAddress port: 4101.
	socket binary.! !

!BrlttyConnection methodsFor: 'network' stamp: 'ThierryGoubier 4/3/2012 21:13'!
disconnect
	"Disconnect the socket."
	
	socket isNil ifTrue: [^self].
	socket close.
	socket atEnd
		ifFalse: [ Socket error: 'brltty connection not closed' ].
	socket destroy.
	socket := nil! !

!BrlttyConnection methodsFor: 'stream out' stamp: 'ThierryGoubier 4/3/2012 21:48'!
flush
	socket flush! !

!BrlttyConnection methodsFor: 'stream in' stamp: 'ThierryGoubier 4/5/2012 22:09'!
next: aSize
	"Read this amount of data from the socket."

	^socket next: aSize! !

!BrlttyConnection methodsFor: 'stream out' stamp: 'ThierryGoubier 4/3/2012 21:33'!
nextPut: type with: data
	"Send a packet with the data and type. Type is a number and data is expected to be either a ByteArray or a ByteString?."

	self nextPutUint32: data size.
	self nextPutUint32: type.
	socket
		nextPutAll: data;
		flush! !

!BrlttyConnection methodsFor: 'stream out' stamp: 'ThierryGoubier 4/3/2012 21:31'!
nextPutUint32: value
	"Send a a value as a four bytes item (positive only)."

	self assert: value positive.
	socket
		nextPut: (value digitAt: 4);
		nextPut: (value digitAt: 3);
		nextPut: (value digitAt: 2);
		nextPut: (value digitAt: 1)! !

!BrlttyConnection methodsFor: 'stream in' stamp: 'ThierryGoubier 4/3/2012 21:16'!
nextUint32
	"Read an uint32 from the stream."

	| d |
	d := socket next: 4.
	^ Integer
		byte1: (d at: 4)
		byte2: (d at: 3)
		byte3: (d at: 2)
		byte4: (d at: 1)! !

!BrlttyConnection methodsFor: 'brltty defines' stamp: 'ThierryGoubier 4/3/2012 21:54'!
packetAcceptKeyRanges
	^ $u asciiValue! !

!BrlttyConnection methodsFor: 'brltty defines' stamp: 'ThierryGoubier 4/3/2012 21:56'!
packetAck
	^ $A asciiValue! !

!BrlttyConnection methodsFor: 'brltty defines' stamp: 'ThierryGoubier 4/3/2012 21:52'!
packetAuth
	^ $a asciiValue! !

!BrlttyConnection methodsFor: 'brltty defines' stamp: 'ThierryGoubier 4/3/2012 21:55'!
packetEnterRawMode
	^ $* asciiValue! !

!BrlttyConnection methodsFor: 'brltty defines' stamp: 'ThierryGoubier 4/3/2012 21:53'!
packetEnterTtyMode
	^ $t asciiValue! !

!BrlttyConnection methodsFor: 'brltty defines' stamp: 'ThierryGoubier 4/3/2012 21:56'!
packetError
	^ $e asciiValue! !

!BrlttyConnection methodsFor: 'brltty defines' stamp: 'ThierryGoubier 4/3/2012 21:56'!
packetException
	^ $E asciiValue! !

!BrlttyConnection methodsFor: 'brltty defines' stamp: 'ThierryGoubier 4/3/2012 21:53'!
packetGetDisplaySize
	^ $s asciiValue! !

!BrlttyConnection methodsFor: 'brltty defines' stamp: 'ThierryGoubier 4/3/2012 21:52'!
packetGetDriverName
	^ $n asciiValue! !

!BrlttyConnection methodsFor: 'brltty defines' stamp: 'ThierryGoubier 4/3/2012 21:54'!
packetIgnoreKeyRanges
	^ $m asciiValue! !

!BrlttyConnection methodsFor: 'brltty defines' stamp: 'ThierryGoubier 4/3/2012 21:54'!
packetKey
	^ $k asciiValue! !

!BrlttyConnection methodsFor: 'brltty defines' stamp: 'ThierryGoubier 4/3/2012 21:55'!
packetLeaveRawMode
	^ $# asciiValue! !

!BrlttyConnection methodsFor: 'brltty defines' stamp: 'ThierryGoubier 4/3/2012 21:53'!
packetLeaveTtyMode
	^ $L asciiValue! !

!BrlttyConnection methodsFor: 'brltty defines' stamp: 'ThierryGoubier 4/3/2012 21:55'!
packetPacket
	^ $p asciiValue! !

!BrlttyConnection methodsFor: 'brltty defines' stamp: 'ThierryGoubier 4/3/2012 21:57'!
packetResumeDriver
	^ $R asciiValue! !

!BrlttyConnection methodsFor: 'brltty defines' stamp: 'ThierryGoubier 4/3/2012 21:53'!
packetSetFocus
	^ $F asciiValue! !

!BrlttyConnection methodsFor: 'brltty defines' stamp: 'ThierryGoubier 4/3/2012 21:56'!
packetSuspendDriver
	^ $S asciiValue! !

!BrlttyConnection methodsFor: 'brltty defines' stamp: 'ThierryGoubier 4/3/2012 21:52'!
packetVersion
	^ $v asciiValue! !

!BrlttyConnection methodsFor: 'brltty defines' stamp: 'ThierryGoubier 4/3/2012 21:55'!
packetWrite
	^ $w asciiValue! !

!BrlttyConnection methodsFor: 'brltty protocol' stamp: 'ThierryGoubier 4/5/2012 22:04'!
readAckOrError
	"Read a response from the brltty server. It may be an ack, ortherwise it may be an error."

	| s t d |
	s := self nextUint32.
	t := self nextUint32.
	s > 0
		ifTrue: [ d := self next: s ].
	t == self packetError
		ifTrue: [ 
			Transcript show: 'Error received'.
			d notNil
				ifTrue: [ Transcript show: 'with data ' , d ].
			Transcript cr ].
	t == self packetException
		ifTrue: [ 
			Transcript show: 'Exception received'.
			d notNil
				ifTrue: [ Transcript show: 'with data ' , d ].
			Transcript cr ].
	t == self packetAck
		ifTrue: [ 
			"Do nothing on an ack, it's all correct and just return the data, it may be an answer."
			^ d ]! !

!BrlttyConnection methodsFor: 'brltty protocol' stamp: 'ThierryGoubier 4/5/2012 21:47'!
readAuth
	"Read an authentification packet, returns the authentification type after reading the packet."

	self assert: self nextUint32 = 4.
	self assert: self nextUint32 = self packetAuth.
	^ self nextUint32! !

!BrlttyConnection methodsFor: 'brltty protocol' stamp: 'ThierryGoubier 4/3/2012 21:59'!
readVersion
	"Send the current version of the protocol (8)."

	self assert: self nextUint32 = 4.
	self assert: self nextUint32 = self packetVersion.
	^ self nextUint32! !

!BrlttyConnection methodsFor: 'brltty protocol' stamp: 'ThierryGoubier 4/5/2012 16:56'!
sendAuthKey: aKey
	"Send the authorisation key."

	self
		nextPutUint32: 4 + aKey size;
		nextPutUint32: (self packetAuth);
		nextPutUint32: (self brlapiAuthKey).
	socket nextPutAll: aKey.
	self flush! !

!BrlttyConnection methodsFor: 'brltty protocol' stamp: 'ThierryGoubier 4/3/2012 21:59'!
sendVersion
	"Send the current version of the protocol (8)."

	self
		nextPutUint32: 4;
		nextPutUint32: (self packetVersion);
		nextPutUint32: 8;
		flush! !

!BrlttyConnection methodsFor: 'accessing' stamp: 'ThierryGoubier 4/3/2012 15:51'!
socket
	^ socket! !

!BrlttyConnection methodsFor: 'accessing' stamp: 'ThierryGoubier 4/3/2012 15:51'!
socket: anObject
	socket := anObject! !
